=== modified file 'client/errname.cc'
--- client/errname.cc	2009-11-27 21:14:56 +0000
+++ client/errname.cc	2009-12-11 21:21:33 +0000
@@ -715,6 +715,7 @@
     { "ER_WARNING_NON_DEFAULT_VALUE_FOR_VIRTUAL_COLUMN",1679 },
     { "ER_UNSUPPORTED_ACTION_ON_VIRTUAL_COLUMN",1680 },
     { "ER_CONST_EXPR_IN_VCOL",1681 },
+    { "ER_NO_PRIMARY_KEY_ON_REPLICATED_TABLE", 1692},
     { 0, 0 }
   };
 

=== modified file 'drizzled/cursor.cc'
--- drizzled/cursor.cc	2009-12-04 20:52:44 +0000
+++ drizzled/cursor.cc	2009-12-11 21:21:33 +0000
@@ -1359,6 +1359,8 @@
   if (table->s->tmp_table || ! replication_services.isActive())
     return false;
 
+  bool result= false;
+
   switch (session->lex->sql_command)
   {
   case SQLCOM_REPLACE:
@@ -1394,7 +1396,7 @@
     else
     {
       if (before_record == NULL)
-        replication_services.insertRecord(session, table);
+        result= replication_services.insertRecord(session, table);
       else
         replication_services.updateRecord(session, table, before_record, after_record);
     }
@@ -1408,7 +1410,7 @@
      * an update.
      */
     if (before_record == NULL)
-      replication_services.insertRecord(session, table);
+      result= replication_services.insertRecord(session, table);
     else
       replication_services.updateRecord(session, table, before_record, after_record);
     break;
@@ -1424,7 +1426,7 @@
     break;
   }
 
-  return false;
+  return result;
 }
 
 int Cursor::ha_external_lock(Session *session, int lock_type)

=== modified file 'drizzled/error.cc'
--- drizzled/error.cc	2009-11-27 21:14:56 +0000
+++ drizzled/error.cc	2009-12-11 19:08:29 +0000
@@ -1413,7 +1413,9 @@
 /* ER_INVALID_TIME_VALUE */
 N_("Received an invalid time value '%s'."),
 /* ER_INVALID_ENUM_VALUE */
-N_("Received an invalid enum value '%s'.")
+N_("Received an invalid enum value '%s'."),
+/* ER_NO_PRIMARY_KEY_ON_REPLICATED_TABLE */
+N_("Tables which are replicated require a primary key.")
 };
 
 const char * error_message(unsigned int code)

=== modified file 'drizzled/error.h'
--- drizzled/error.h	2009-11-27 21:14:56 +0000
+++ drizzled/error.h	2009-12-11 19:08:29 +0000
@@ -721,7 +721,8 @@
   ER_ARGUMENT_OUT_OF_RANGE,
   ER_INVALID_TIME_VALUE,
   ER_INVALID_ENUM_VALUE,
-  ER_ERROR_LAST= ER_INVALID_ENUM_VALUE
+  ER_NO_PRIMARY_KEY_ON_REPLICATED_TABLE,
+  ER_ERROR_LAST= ER_NO_PRIMARY_KEY_ON_REPLICATED_TABLE
 };
 
 #ifdef  __cplusplus
@@ -729,4 +730,3 @@
 #endif
 
 #endif /* DRIZZLED_ERROR_H */
-

=== modified file 'drizzled/replication_services.cc'
--- drizzled/replication_services.cc	2009-12-04 19:50:38 +0000
+++ drizzled/replication_services.cc	2009-12-15 17:54:03 +0000
@@ -58,6 +58,7 @@
 #include "drizzled/message/table.pb.h"
 #include "drizzled/gettext.h"
 #include "drizzled/session.h"
+#include "drizzled/error.h"
 
 #include <vector>
 
@@ -328,9 +329,20 @@
 }
 
 message::Statement &ReplicationServices::getInsertStatement(Session *in_session,
-                                                            Table *in_table) const
+                                                                 Table *in_table) const
 {
   message::Statement *statement= in_session->getStatementMessage();
+  /*
+   * We check to see if the current Statement message is of type INSERT.
+   * If it is not, we finalize the current Statement and ensure a new
+   * InsertStatement is created.
+   */
+  if (statement != NULL &&
+      statement->type() != message::Statement::INSERT)
+  {
+    finalizeStatement(*statement, in_session);
+    statement= in_session->getStatementMessage();
+  }
 
   if (statement == NULL)
   {
@@ -384,10 +396,23 @@
   }
 }
 
-void ReplicationServices::insertRecord(Session *in_session, Table *in_table)
+bool ReplicationServices::insertRecord(Session *in_session, Table *in_table)
 {
   if (! is_active)
-    return;
+    return false;
+  /**
+   * We do this check here because we don't want to even create a 
+   * statement if there isn't a primary key on the table...
+   *
+   * @todo
+   *
+   * Multi-column primary keys are handled how exactly?
+   */
+  if (in_table->s->primary_key == MAX_KEY)
+  {
+    my_error(ER_NO_PRIMARY_KEY_ON_REPLICATED_TABLE, MYF(0));
+    return true;
+  }
 
   message::Statement &statement= getInsertStatement(in_session, in_table);
 
@@ -411,6 +436,7 @@
     record->add_insert_value(string_value->c_ptr(), string_value->length());
     string_value->free();
   }
+  return false;
 }
 
 message::Statement &ReplicationServices::getUpdateStatement(Session *in_session,
@@ -419,6 +445,17 @@
                                                             const unsigned char *new_record) const
 {
   message::Statement *statement= in_session->getStatementMessage();
+  /*
+   * We check to see if the current Statement message is of type UPDATE.
+   * If it is not, we finalize the current Statement and ensure a new
+   * UpdateStatement is created.
+   */
+  if (statement != NULL &&
+      statement->type() != message::Statement::UPDATE)
+  {
+    finalizeStatement(*statement, in_session);
+    statement= in_session->getStatementMessage();
+  }
 
   if (statement == NULL)
   {
@@ -460,8 +497,6 @@
 
   Field *current_field;
   Field **table_fields= in_table->field;
-  String *string_value= new (in_session->mem_root) String(ReplicationServices::DEFAULT_RECORD_SIZE);
-  string_value->set_charset(system_charset_info);
 
   message::FieldMetadata *field_metadata;
 
@@ -589,6 +624,17 @@
                                                             Table *in_table) const
 {
   message::Statement *statement= in_session->getStatementMessage();
+  /*
+   * We check to see if the current Statement message is of type DELETE.
+   * If it is not, we finalize the current Statement and ensure a new
+   * DeleteStatement is created.
+   */
+  if (statement != NULL &&
+      statement->type() != message::Statement::DELETE)
+  {
+    finalizeStatement(*statement, in_session);
+    statement= in_session->getStatementMessage();
+  }
 
   if (statement == NULL)
   {

=== modified file 'drizzled/replication_services.h'
--- drizzled/replication_services.h	2009-12-04 16:59:27 +0000
+++ drizzled/replication_services.h	2009-12-11 21:21:33 +0000
@@ -294,8 +294,11 @@
    *
    * @param Pointer to the Session which has inserted a record
    * @param Pointer to the Table containing insert information
+   *
+   * Grr, returning "true" here on error because of the cursor
+   * reversed bool return crap...fix that.
    */
-  void insertRecord(Session *in_session, Table *in_table);
+  bool insertRecord(Session *in_session, Table *in_table);
   /**
    * Creates a new UpdateRecord GPB message and pushes it to
    * replicators.

=== modified file 'plugin/transaction_log/tests/r/auto_commit.result'
--- plugin/transaction_log/tests/r/auto_commit.result	2009-10-15 18:20:11 +0000
+++ plugin/transaction_log/tests/r/auto_commit.result	2009-12-11 21:28:10 +0000
@@ -1,7 +1,7 @@
 DROP TABLE IF EXISTS t1;
 SET AUTOCOMMIT= 0;
 CREATE TABLE t1 (
-id INT NOT NULL
+id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 INSERT INTO t1 VALUES (1, "I love testing.");
@@ -9,20 +9,20 @@
 DROP TABLE t1;
 SET AUTOCOMMIT= 1;
 CREATE TABLE t1 (
-id INT NOT NULL
+id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 INSERT INTO t1 VALUES (1, "I love testing.");
 INSERT INTO t1 VALUES (2, "I hate testing.");
 DROP TABLE t1;
 DROP TABLE IF EXISTS `t1`;
-CREATE TABLE t1 ( id INT NOT NULL , padding VARCHAR(200) NOT NULL );
+CREATE TABLE t1 ( id INT NOT NULL PRIMARY KEY , padding VARCHAR(200) NOT NULL );
 START TRANSACTION;
 INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (1,'I love testing.');
 INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (2,'I hate testing.');
 COMMIT;
 DROP TABLE `t1`;
-CREATE TABLE t1 ( id INT NOT NULL , padding VARCHAR(200) NOT NULL );
+CREATE TABLE t1 ( id INT NOT NULL PRIMARY KEY , padding VARCHAR(200) NOT NULL );
 INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (1,'I love testing.');
 INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (2,'I hate testing.');
 DROP TABLE `t1`;

=== modified file 'plugin/transaction_log/tests/r/filtered_replicator.result'
--- plugin/transaction_log/tests/r/filtered_replicator.result	2009-12-04 19:50:38 +0000
+++ plugin/transaction_log/tests/r/filtered_replicator.result	2009-12-11 21:30:49 +0000
@@ -1,27 +1,27 @@
 set global filtered_replicator_filteredtables=t2;
 DROP TABLE IF EXISTS t1;
 CREATE TABLE t1 (
-id INT NOT NULL
+id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 INSERT INTO t1 VALUES (1, "I love testing.");
 INSERT INTO t1 VALUES (2, "I hate testing.");
 DROP TABLE IF EXISTS t1;
 CREATE TABLE t1 (
-id INT NOT NULL
+id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 INSERT INTO t1 VALUES (1, "I love testing."), (2, "I hate testing.");
 DROP TABLE IF EXISTS t1;
 DROP TABLE IF EXISTS t2;
 CREATE TABLE t1 (
-id INT NOT NULL
+id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 INSERT INTO t1 VALUES (1, "I love testing.");
 INSERT INTO t1 VALUES (2, "I hate testing.");
 CREATE TABLE t2 (
-id INT NOT NULL
+id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 INSERT INTO t2 SELECT * FROM t1;
@@ -107,18 +107,18 @@
 UPDATE t1 SET id = 4 WHERE id = 2;
 DROP TABLE t1;
 DROP TABLE IF EXISTS `t1`;
-CREATE TABLE t1 ( id INT NOT NULL , padding VARCHAR(200) NOT NULL );
+CREATE TABLE t1 ( id INT NOT NULL PRIMARY KEY , padding VARCHAR(200) NOT NULL );
 INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (1,'I love testing.');
 INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (2,'I hate testing.');
 DROP TABLE IF EXISTS `t1`;
-CREATE TABLE t1 ( id INT NOT NULL , padding VARCHAR(200) NOT NULL );
+CREATE TABLE t1 ( id INT NOT NULL PRIMARY KEY , padding VARCHAR(200) NOT NULL );
 START TRANSACTION;
 INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (1,'I love testing.');
 INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (2,'I hate testing.');
 COMMIT;
 DROP TABLE IF EXISTS `t1`;
 DROP TABLE IF EXISTS `t2`;
-CREATE TABLE t1 ( id INT NOT NULL , padding VARCHAR(200) NOT NULL );
+CREATE TABLE t1 ( id INT NOT NULL PRIMARY KEY , padding VARCHAR(200) NOT NULL );
 INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (1,'I love testing.');
 INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (2,'I hate testing.');
 DROP TABLE IF EXISTS `t1`;

=== modified file 'plugin/transaction_log/tests/r/information_schema.result'
--- plugin/transaction_log/tests/r/information_schema.result	2009-12-01 00:02:53 +0000
+++ plugin/transaction_log/tests/r/information_schema.result	2009-12-11 19:08:29 +0000
@@ -1,6 +1,6 @@
 DROP TABLE IF EXISTS t1;
 CREATE TABLE t1 (
-id INT NOT NULL
+id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 INSERT INTO t1 VALUES (1, "I love testing.");

=== modified file 'plugin/transaction_log/tests/r/insert.result'
--- plugin/transaction_log/tests/r/insert.result	2009-10-15 18:20:11 +0000
+++ plugin/transaction_log/tests/r/insert.result	2009-12-11 19:08:29 +0000
@@ -1,12 +1,12 @@
 DROP TABLE IF EXISTS t1;
 CREATE TABLE t1 (
-id INT NOT NULL
+id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 INSERT INTO t1 VALUES (1, "I love testing.");
 INSERT INTO t1 VALUES (2, "I hate testing.");
 DROP TABLE IF EXISTS `t1`;
-CREATE TABLE t1 ( id INT NOT NULL , padding VARCHAR(200) NOT NULL );
+CREATE TABLE t1 ( id INT NOT NULL PRIMARY KEY , padding VARCHAR(200) NOT NULL );
 INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (1,'I love testing.');
 INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (2,'I hate testing.');
 SET GLOBAL transaction_log_truncate_debug= true;

=== modified file 'plugin/transaction_log/tests/r/insert_multi.result'
--- plugin/transaction_log/tests/r/insert_multi.result	2009-10-15 18:20:11 +0000
+++ plugin/transaction_log/tests/r/insert_multi.result	2009-12-11 21:30:49 +0000
@@ -1,11 +1,11 @@
 DROP TABLE IF EXISTS t1;
 CREATE TABLE t1 (
-id INT NOT NULL
+id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 INSERT INTO t1 VALUES (1, "I love testing."), (2, "I hate testing.");
 DROP TABLE IF EXISTS `t1`;
-CREATE TABLE t1 ( id INT NOT NULL , padding VARCHAR(200) NOT NULL );
+CREATE TABLE t1 ( id INT NOT NULL PRIMARY KEY , padding VARCHAR(200) NOT NULL );
 START TRANSACTION;
 INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (1,'I love testing.');
 INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (2,'I hate testing.');

=== modified file 'plugin/transaction_log/tests/r/insert_select.result'
--- plugin/transaction_log/tests/r/insert_select.result	2009-10-15 18:20:11 +0000
+++ plugin/transaction_log/tests/r/insert_select.result	2009-12-11 21:30:49 +0000
@@ -1,22 +1,22 @@
 DROP TABLE IF EXISTS t1;
 DROP TABLE IF EXISTS t2;
 CREATE TABLE t1 (
-id INT NOT NULL
+id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 INSERT INTO t1 VALUES (1, "I love testing.");
 INSERT INTO t1 VALUES (2, "I hate testing.");
 CREATE TABLE t2 (
-id INT NOT NULL
+id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 INSERT INTO t2 SELECT * FROM t1;
 DROP TABLE IF EXISTS `t1`;
 DROP TABLE IF EXISTS `t2`;
-CREATE TABLE t1 ( id INT NOT NULL , padding VARCHAR(200) NOT NULL );
+CREATE TABLE t1 ( id INT NOT NULL PRIMARY KEY , padding VARCHAR(200) NOT NULL );
 INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (1,'I love testing.');
 INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (2,'I hate testing.');
-CREATE TABLE t2 ( id INT NOT NULL , padding VARCHAR(200) NOT NULL );
+CREATE TABLE t2 ( id INT NOT NULL PRIMARY KEY , padding VARCHAR(200) NOT NULL );
 START TRANSACTION;
 INSERT INTO `test`.`t2` (`id`,`padding`) VALUES (1,'I love testing.');
 INSERT INTO `test`.`t2` (`id`,`padding`) VALUES (2,'I hate testing.');

=== modified file 'plugin/transaction_log/tests/r/no_modification.result'
--- plugin/transaction_log/tests/r/no_modification.result	2009-10-29 18:58:54 +0000
+++ plugin/transaction_log/tests/r/no_modification.result	2009-12-14 20:30:26 +0000
@@ -1,10 +1,11 @@
 DROP TABLE IF EXISTS t1;
 CREATE TABLE t1 (
-id INT NOT NULL
+id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 INSERT INTO t1 VALUES (1, "I love testing.");
 INSERT INTO t1 VALUES (2, "I hate testing.");
+UPDATE t1 SET id=1 WHERE padding='I love testing.';
 START TRANSACTION;
 SELECT * FROM t1 WHERE id = 1;
 id	padding
@@ -14,7 +15,7 @@
 2	I hate testing.
 COMMIT;
 DROP TABLE IF EXISTS `t1`;
-CREATE TABLE t1 ( id INT NOT NULL , padding VARCHAR(200) NOT NULL );
+CREATE TABLE t1 ( id INT NOT NULL PRIMARY KEY , padding VARCHAR(200) NOT NULL );
 INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (1,'I love testing.');
 INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (2,'I hate testing.');
 SET GLOBAL transaction_log_truncate_debug= true;

=== added file 'plugin/transaction_log/tests/r/no_primary_key.result'
--- plugin/transaction_log/tests/r/no_primary_key.result	1970-01-01 00:00:00 +0000
+++ plugin/transaction_log/tests/r/no_primary_key.result	2009-12-14 21:49:13 +0000
@@ -0,0 +1,12 @@
+DROP TABLE IF EXISTS t1;
+CREATE TABLE t1 (
+id INT NOT NULL
+, padding VARCHAR(200) NOT NULL
+);
+INSERT INTO t1 VALUES (1, "I love testing.");
+ERROR HY000: Tables which are replicated require a primary key.
+DROP TABLE t1;
+DROP TABLE IF EXISTS `t1`;
+CREATE TABLE t1 ( id INT NOT NULL , padding VARCHAR(200) NOT NULL );
+DROP TABLE `t1`;
+SET GLOBAL transaction_log_truncate_debug= true;

=== added file 'plugin/transaction_log/tests/r/select_for_update.result'
--- plugin/transaction_log/tests/r/select_for_update.result	1970-01-01 00:00:00 +0000
+++ plugin/transaction_log/tests/r/select_for_update.result	2009-12-09 01:22:20 +0000
@@ -0,0 +1,32 @@
+DROP TABLE IF EXISTS t1;
+CREATE TABLE t1 (
+id INT NOT NULL
+, padding VARCHAR(200) NOT NULL
+, PRIMARY KEY (id)
+);
+INSERT INTO t1 VALUES (1, "I love testing.");
+INSERT INTO t1 VALUES (3, "I hate testing.");
+INSERT INTO t1 VALUES (5, "I still hate testing.");
+START TRANSACTION;
+SELECT id FROM t1 FOR UPDATE;
+id
+1
+3
+5
+UPDATE t1 SET id=1000 WHERE padding='I love testing.';
+ERROR HY000: Lock wait timeout exceeded; try restarting transaction
+UPDATE t1 SET id=id + 1;
+COMMIT;
+DROP TABLE t1;
+DROP TABLE IF EXISTS `t1`;
+CREATE TABLE t1 ( id INT NOT NULL , padding VARCHAR(200) NOT NULL , PRIMARY KEY (id) );
+INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (1,'I love testing.');
+INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (3,'I hate testing.');
+INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (5,'I still hate testing.');
+START TRANSACTION;
+UPDATE `test`.`t1` SET `id`=2 WHERE `id`=1;
+UPDATE `test`.`t1` SET `id`=4 WHERE `id`=3;
+UPDATE `test`.`t1` SET `id`=6 WHERE `id`=5;
+COMMIT;
+DROP TABLE `t1`;
+SET GLOBAL transaction_log_truncate_debug= true;

=== added file 'plugin/transaction_log/tests/r/sync_method_every_write.result'
--- plugin/transaction_log/tests/r/sync_method_every_write.result	1970-01-01 00:00:00 +0000
+++ plugin/transaction_log/tests/r/sync_method_every_write.result	2009-12-11 18:39:37 +0000
@@ -0,0 +1,8 @@
+SELECT * FROM INFORMATION_SCHEMA.GLOBAL_VARIABLES
+WHERE Variable_name LIKE 'transaction_log%';
+VARIABLE_NAME	VARIABLE_VALUE
+TRANSACTION_LOG_ENABLE	ON
+TRANSACTION_LOG_ENABLE_CHECKSUM	OFF
+TRANSACTION_LOG_LOG_FILE	transaction.log
+TRANSACTION_LOG_SYNC_METHOD	1
+TRANSACTION_LOG_TRUNCATE_DEBUG	OFF

=== modified file 'plugin/transaction_log/tests/r/temp_tables.result'
--- plugin/transaction_log/tests/r/temp_tables.result	2009-11-20 01:08:36 +0000
+++ plugin/transaction_log/tests/r/temp_tables.result	2009-12-14 21:49:13 +0000
@@ -1,6 +1,6 @@
 DROP TABLE IF EXISTS t1, t2;
 CREATE TEMPORARY TABLE t1 (
-id INT NOT NULL
+id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 INSERT INTO t1 VALUES (1, "I love testing.");
@@ -11,7 +11,7 @@
 DROP TABLE t2;
 DROP TABLE IF EXISTS `t1`;
 DROP TABLE IF EXISTS `t2`;
-CREATE TABLE `t2` (   `id` int NOT NULL,   `padding` varchar(200) NOT NULL ) ENGINE=InnoDB;
+CREATE TABLE `t2` (   `id` int NOT NULL,   `padding` varchar(200) NOT NULL,   PRIMARY KEY (`id`) ) ENGINE=InnoDB;
 START TRANSACTION;
 INSERT INTO `test`.`t2` (`id`,`padding`) VALUES (1,'I love testing.');
 INSERT INTO `test`.`t2` (`id`,`padding`) VALUES (2,'I hate testing.');

=== modified file 'plugin/transaction_log/tests/r/truncate.result'
--- plugin/transaction_log/tests/r/truncate.result	2009-12-04 02:24:50 +0000
+++ plugin/transaction_log/tests/r/truncate.result	2009-12-14 21:49:13 +0000
@@ -1,13 +1,13 @@
 DROP TABLE IF EXISTS t1;
 CREATE TABLE t1 (
-id INT NOT NULL
+id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 INSERT INTO t1 VALUES (1, "I love testing.");
 INSERT INTO t1 VALUES (2, "I hate testing.");
 TRUNCATE TABLE t1;
 DROP TABLE IF EXISTS `t1`;
-CREATE TABLE t1 ( id INT NOT NULL , padding VARCHAR(200) NOT NULL );
+CREATE TABLE t1 ( id INT NOT NULL PRIMARY KEY , padding VARCHAR(200) NOT NULL );
 INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (1,'I love testing.');
 INSERT INTO `test`.`t1` (`id`,`padding`) VALUES (2,'I hate testing.');
 TRUNCATE TABLE `test`.`t1`;

=== modified file 'plugin/transaction_log/tests/r/truncate_log.result'
--- plugin/transaction_log/tests/r/truncate_log.result	2009-10-07 23:59:47 +0000
+++ plugin/transaction_log/tests/r/truncate_log.result	2009-12-11 19:08:29 +0000
@@ -1,6 +1,6 @@
 DROP TABLE IF EXISTS t1;
 CREATE TABLE t1 (
-id INT NOT NULL
+id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 INSERT INTO t1 VALUES (1, "I love testing.");

=== modified file 'plugin/transaction_log/tests/r/udf_print_transaction_message.result'
--- plugin/transaction_log/tests/r/udf_print_transaction_message.result	2009-11-20 20:22:37 +0000
+++ plugin/transaction_log/tests/r/udf_print_transaction_message.result	2009-12-11 19:08:29 +0000
@@ -1,6 +1,6 @@
 DROP TABLE IF EXISTS t1;
 CREATE TABLE t1 (
-id INT NOT NULL
+id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 INSERT INTO t1 VALUES (1, "I love testing.");

=== modified file 'plugin/transaction_log/tests/t/auto_commit.inc'
--- plugin/transaction_log/tests/t/auto_commit.inc	2009-10-07 23:59:47 +0000
+++ plugin/transaction_log/tests/t/auto_commit.inc	2009-12-11 21:28:10 +0000
@@ -9,7 +9,7 @@
 SET AUTOCOMMIT= 0;
 
 CREATE TABLE t1 (
-  id INT NOT NULL
+  id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 
@@ -24,7 +24,7 @@
 SET AUTOCOMMIT= 1;
 
 CREATE TABLE t1 (
-  id INT NOT NULL
+  id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 

=== modified file 'plugin/transaction_log/tests/t/insert.inc'
--- plugin/transaction_log/tests/t/insert.inc	2009-07-02 05:08:33 +0000
+++ plugin/transaction_log/tests/t/insert.inc	2009-12-11 19:08:29 +0000
@@ -1,5 +1,5 @@
 # 
-# Simple test of the serial event log for single INSERT statements
+# Simple test of the transaction log for single INSERT statements
 # 
 # We create a table and insert some records
 # into it.
@@ -12,7 +12,7 @@
 --enable_warnings
 
 CREATE TABLE t1 (
-  id INT NOT NULL
+  id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 

=== modified file 'plugin/transaction_log/tests/t/insert_multi.inc'
--- plugin/transaction_log/tests/t/insert_multi.inc	2009-07-29 23:25:20 +0000
+++ plugin/transaction_log/tests/t/insert_multi.inc	2009-12-11 21:30:49 +0000
@@ -12,7 +12,7 @@
 --enable_warnings
 
 CREATE TABLE t1 (
-  id INT NOT NULL
+  id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 

=== modified file 'plugin/transaction_log/tests/t/insert_select.inc'
--- plugin/transaction_log/tests/t/insert_select.inc	2009-07-29 23:25:20 +0000
+++ plugin/transaction_log/tests/t/insert_select.inc	2009-12-11 21:30:49 +0000
@@ -15,7 +15,7 @@
 --enable_warnings
 
 CREATE TABLE t1 (
-  id INT NOT NULL
+  id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 
@@ -23,7 +23,7 @@
 INSERT INTO t1 VALUES (2, "I hate testing.");
 
 CREATE TABLE t2 (
-  id INT NOT NULL
+  id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 

=== modified file 'plugin/transaction_log/tests/t/no_modification.inc'
--- plugin/transaction_log/tests/t/no_modification.inc	2009-10-29 18:58:54 +0000
+++ plugin/transaction_log/tests/t/no_modification.inc	2009-12-14 20:30:26 +0000
@@ -1,6 +1,7 @@
 # 
 # Simple test of the transaction log for when a START TRANSACTION ... COMMIT
-# is issued, but no data modifications actually occurred.
+# is issued, but no data modifications actually occurred. Also tested is
+# a update that does not update anything. 
 # 
 # This situation occurs in, for instance, sysbench, which wraps even SELECTs
 # in a transaction.
@@ -11,13 +12,15 @@
 --enable_warnings
 
 CREATE TABLE t1 (
-  id INT NOT NULL
+  id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 
 INSERT INTO t1 VALUES (1, "I love testing.");
 INSERT INTO t1 VALUES (2, "I hate testing.");
 
+UPDATE t1 SET id=1 WHERE padding='I love testing.';
+
 START TRANSACTION;
 
 SELECT * FROM t1 WHERE id = 1;

=== added file 'plugin/transaction_log/tests/t/no_primary_key-master.opt'
--- plugin/transaction_log/tests/t/no_primary_key-master.opt	1970-01-01 00:00:00 +0000
+++ plugin/transaction_log/tests/t/no_primary_key-master.opt	2009-12-11 21:21:33 +0000
@@ -0,0 +1,1 @@
+--default-replicator-enable --transaction-log-enable --scheduler=multi_thread

=== added file 'plugin/transaction_log/tests/t/no_primary_key.inc'
--- plugin/transaction_log/tests/t/no_primary_key.inc	1970-01-01 00:00:00 +0000
+++ plugin/transaction_log/tests/t/no_primary_key.inc	2009-12-18 18:03:15 +0000
@@ -0,0 +1,25 @@
+# 
+# Simple test of the transaction log and the errors
+# which should be produced when a table is inserted
+# into when the transaction log is enabled but the
+# table does not have primary keys.
+# 
+# We create a table without a primary key and insert 
+# a record into it.  An error should be produced.
+#
+
+--disable_warnings
+DROP TABLE IF EXISTS t1;
+--enable_warnings
+
+CREATE TABLE t1 (
+  id INT NOT NULL
+, padding VARCHAR(200) NOT NULL
+);
+
+# The below should produce an error because the CREATE TABLE
+# above does not have a primary key.
+--error ER_NO_PRIMARY_KEY_ON_REPLICATED_TABLE
+INSERT INTO t1 VALUES (1, "I love testing.");
+
+DROP TABLE t1;

=== added file 'plugin/transaction_log/tests/t/no_primary_key.test'
--- plugin/transaction_log/tests/t/no_primary_key.test	1970-01-01 00:00:00 +0000
+++ plugin/transaction_log/tests/t/no_primary_key.test	2009-12-14 21:49:13 +0000
@@ -0,0 +1,14 @@
+#
+# Tests that tables without primary keys do not have
+# INSERT statements logged to the transaction log
+#
+
+# Populate log with some records...
+--source ../plugin/transaction_log/tests/t/no_primary_key.inc
+
+# Read in the transaction.log.
+
+--exec ../drizzled/message/transaction_reader var/master-data/transaction.log
+
+# Truncate the log file to reset for the next test
+--source ../plugin/transaction_log/tests/t/truncate_log.inc

=== added file 'plugin/transaction_log/tests/t/select_for_update-master.opt'
--- plugin/transaction_log/tests/t/select_for_update-master.opt	1970-01-01 00:00:00 +0000
+++ plugin/transaction_log/tests/t/select_for_update-master.opt	2009-12-07 03:59:58 +0000
@@ -0,0 +1,1 @@
+--default-replicator-enable --transaction-log-enable --scheduler=multi_thread

=== added file 'plugin/transaction_log/tests/t/select_for_update.inc'
--- plugin/transaction_log/tests/t/select_for_update.inc	1970-01-01 00:00:00 +0000
+++ plugin/transaction_log/tests/t/select_for_update.inc	2009-12-09 01:22:20 +0000
@@ -0,0 +1,37 @@
+# 
+# Test to verify updates from SELECT FOR UPDATE are handled 
+# properly, an update is not put in the log for a update that
+# times out. 
+# 
+
+--disable_warnings
+DROP TABLE IF EXISTS t1;
+--enable_warnings
+
+CREATE TABLE t1 (
+  id INT NOT NULL
+, padding VARCHAR(200) NOT NULL
+, PRIMARY KEY (id)
+);
+
+INSERT INTO t1 VALUES (1, "I love testing.");
+INSERT INTO t1 VALUES (3, "I hate testing.");
+INSERT INTO t1 VALUES (5, "I still hate testing.");
+
+connect (conn1, localhost, root,,);
+connect (conn2, localhost, root,,);
+
+connection conn1;
+START TRANSACTION;
+SELECT id FROM t1 FOR UPDATE;
+
+connection conn2;
+# This should timeout and not appear in the transaction log
+--error ER_LOCK_WAIT_TIMEOUT
+UPDATE t1 SET id=1000 WHERE padding='I love testing.';
+
+connection conn1;
+UPDATE t1 SET id=id + 1;
+COMMIT;
+
+DROP TABLE t1;

=== added file 'plugin/transaction_log/tests/t/select_for_update.test'
--- plugin/transaction_log/tests/t/select_for_update.test	1970-01-01 00:00:00 +0000
+++ plugin/transaction_log/tests/t/select_for_update.test	2009-12-09 01:37:14 +0000
@@ -0,0 +1,13 @@
+#
+# Tests SELECT FOR UPDATE 
+#
+
+# Populate log with some records...
+--source ../plugin/transaction_log/tests/t/select_for_update.inc
+
+# Read in the transaction.log.
+
+--exec ../drizzled/message/transaction_reader var/master-data/transaction.log
+
+# Truncate the log file to reset for the next test
+--source ../plugin/transaction_log/tests/t/truncate_log.inc

=== added file 'plugin/transaction_log/tests/t/sync_method_every_write-master.opt'
--- plugin/transaction_log/tests/t/sync_method_every_write-master.opt	1970-01-01 00:00:00 +0000
+++ plugin/transaction_log/tests/t/sync_method_every_write-master.opt	2009-12-11 18:39:37 +0000
@@ -0,0 +1,1 @@
+--default-replicator-enable --transaction-log-enable --transaction-log-sync-method=1 --scheduler=multi_thread

=== added file 'plugin/transaction_log/tests/t/sync_method_every_write.test'
--- plugin/transaction_log/tests/t/sync_method_every_write.test	1970-01-01 00:00:00 +0000
+++ plugin/transaction_log/tests/t/sync_method_every_write.test	2009-12-11 18:39:37 +0000
@@ -0,0 +1,12 @@
+#
+# Tests the behvaiour of the transaction-log-sync-method
+# configuration option.  At this point, the only thing we
+# can check is that the options appear in the INFORMATION_SCHEMA.VARIABLES
+# view, since I can't think of a way to test fsync() behaviour
+# properly through the test suite...it might be that we just do this
+# basic variables test and rely on LCOV code coverage to check
+# the sync methods?
+#
+
+SELECT * FROM INFORMATION_SCHEMA.GLOBAL_VARIABLES
+WHERE Variable_name LIKE 'transaction_log%';

=== modified file 'plugin/transaction_log/tests/t/temp_tables.inc'
--- plugin/transaction_log/tests/t/temp_tables.inc	2009-10-16 21:34:21 +0000
+++ plugin/transaction_log/tests/t/temp_tables.inc	2009-12-14 21:49:13 +0000
@@ -8,7 +8,7 @@
 --enable_warnings
 
 CREATE TEMPORARY TABLE t1 (
-  id INT NOT NULL
+  id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 

=== modified file 'plugin/transaction_log/tests/t/truncate.inc'
--- plugin/transaction_log/tests/t/truncate.inc	2009-07-02 05:08:33 +0000
+++ plugin/transaction_log/tests/t/truncate.inc	2009-12-14 21:49:13 +0000
@@ -12,7 +12,7 @@
 --enable_warnings
 
 CREATE TABLE t1 (
-  id INT NOT NULL
+  id INT NOT NULL PRIMARY KEY
 , padding VARCHAR(200) NOT NULL
 );
 

=== modified file 'plugin/transaction_log/transaction_log.cc'
--- plugin/transaction_log/transaction_log.cc	2009-12-01 17:47:52 +0000
+++ plugin/transaction_log/transaction_log.cc	2009-12-11 18:39:37 +0000
@@ -115,6 +115,15 @@
  * each written Transaction message?
  */
 static bool sysvar_transaction_log_checksum_enabled= false;
+/**
+ * Numeric option controlling the sync/flush behaviour of the transaction
+ * log.  Options are:
+ *
+ * TransactionLog::SYNC_METHOD_OS == 0            ... let OS do sync'ing
+ * TransactionLog::SYNC_METHOD_EVERY_WRITE == 1   ... sync on every write
+ * TransactionLog::SYNC_METHOD_EVERY_SECOND == 2  ... sync at most once a second
+ */
+static uint32_t sysvar_transaction_log_sync_method= 0;
 
 /** Views defined in info_schema.cc */
 extern plugin::InfoSchemaTable *transaction_log_view;
@@ -188,7 +197,6 @@
                       message and trailing checksum to */
   uint8_t *orig_buffer;
 
-  int error_code;
   size_t message_byte_length= to_apply.ByteSize();
   ssize_t written;
   off_t cur_offset;
@@ -287,7 +295,7 @@
   }
   free(orig_buffer);
 
-  error_code= my_sync(log_file, 0);
+  int error_code= syncLogFile();
 
   transaction_log_index->addEntry(TransactionLogEntry(ReplicationServices::TRANSACTION,
                                                      cur_offset,
@@ -303,6 +311,28 @@
   }
 }
 
+int TransactionLog::syncLogFile()
+{
+  switch (sysvar_transaction_log_sync_method)
+  {
+  case SYNC_METHOD_EVERY_WRITE:
+    return my_sync(log_file, 0);
+  case SYNC_METHOD_EVERY_SECOND:
+    {
+      time_t now_time= time(NULL);
+      if (last_sync_time <= (now_time - 1))
+      {
+        last_sync_time= now_time;
+        return my_sync(log_file, 0);
+      }
+      return 0;
+    }
+  case SYNC_METHOD_OS:
+  default:
+    return 0;
+  }
+}
+
 const string &TransactionLog::getLogFilename()
 {
   return log_file_name;
@@ -489,42 +519,56 @@
 }
 
 static DRIZZLE_SYSVAR_BOOL(enable,
-                          sysvar_transaction_log_enabled,
-                          PLUGIN_VAR_NOCMDARG,
-                          N_("Enable transaction log"),
-                          NULL, /* check func */
-                          NULL, /* update func */
-                          false /* default */);
+                           sysvar_transaction_log_enabled,
+                           PLUGIN_VAR_NOCMDARG,
+                           N_("Enable transaction log"),
+                           NULL, /* check func */
+                           NULL, /* update func */
+                           false /* default */);
 
 static DRIZZLE_SYSVAR_BOOL(truncate_debug,
-                          sysvar_transaction_log_truncate_debug,
-                          PLUGIN_VAR_NOCMDARG,
-                          N_("DEBUGGING - Truncate transaction log"),
-                          NULL, /* check func */
-                          set_truncate_debug, /* update func */
-                          false /* default */);
+                           sysvar_transaction_log_truncate_debug,
+                           PLUGIN_VAR_NOCMDARG,
+                           N_("DEBUGGING - Truncate transaction log"),
+                           NULL, /* check func */
+                           set_truncate_debug, /* update func */
+                           false /* default */);
 
 static DRIZZLE_SYSVAR_STR(log_file,
                           sysvar_transaction_log_file,
                           PLUGIN_VAR_READONLY,
-                          N_("Path to the file to use for transaction log."),
+                          N_("Path to the file to use for transaction log"),
                           NULL, /* check func */
                           NULL, /* update func*/
                           DEFAULT_LOG_FILE_PATH /* default */);
 
 static DRIZZLE_SYSVAR_BOOL(enable_checksum,
-                          sysvar_transaction_log_checksum_enabled,
-                          PLUGIN_VAR_NOCMDARG,
-                          N_("Enable CRC32 Checksumming"),
-                          NULL, /* check func */
-                          NULL, /* update func */
-                          false /* default */);
+                           sysvar_transaction_log_checksum_enabled,
+                           PLUGIN_VAR_NOCMDARG,
+                           N_("Enable CRC32 Checksumming of each written transaction log entry"),
+                           NULL, /* check func */
+                           NULL, /* update func */
+                           false /* default */);
+
+static DRIZZLE_SYSVAR_UINT(sync_method,
+                           sysvar_transaction_log_sync_method,
+                           PLUGIN_VAR_OPCMDARG,
+                           N_("0 == rely on operating system to sync log file (default), "
+                              "1 == sync file at each transaction write, "
+                              "2 == sync log file once per second"),
+                           NULL, /* check func */
+                           NULL, /* update func */
+                           0, /* default */
+                           0,
+                           2,
+                           0);
 
 static drizzle_sys_var* system_variables[]= {
   DRIZZLE_SYSVAR(enable),
   DRIZZLE_SYSVAR(truncate_debug),
   DRIZZLE_SYSVAR(log_file),
   DRIZZLE_SYSVAR(enable_checksum),
+  DRIZZLE_SYSVAR(sync_method),
   NULL
 };
 

=== modified file 'plugin/transaction_log/transaction_log.h'
--- plugin/transaction_log/transaction_log.h	2009-11-26 02:50:58 +0000
+++ plugin/transaction_log/transaction_log.h	2009-12-11 18:39:37 +0000
@@ -71,6 +71,9 @@
     ONLINE,
     WRITING
   };
+  static const uint32_t SYNC_METHOD_OS= 0; ///< Rely on operating system to sync log file
+  static const uint32_t SYNC_METHOD_EVERY_WRITE= 1; //< Sync on every write to the log file
+  static const uint32_t SYNC_METHOD_EVERY_SECOND= 2; ///< Sync no more than once a second
 public:
   TransactionLog(std::string name_arg,
                  const std::string &in_log_file_path,
@@ -169,6 +172,16 @@
    * Clears the current error message
    */
   void clearError();
+  /**
+   * Helper method which synchronizes/flushes the transaction log file
+   * according to the transaction_log_sync_method system variable
+   *
+   * @retval
+   *   0 == Success
+   * @retval
+   *   >0 == Failure. Error code.
+   */
+  int syncLogFile();
 
   int log_file; ///< Handle for our log file
   Status state; ///< The state the log is in
@@ -178,6 +191,7 @@
   drizzled::atomic<off_t> log_offset; ///< Offset in log file where log will write next command
   bool has_error; ///< Is the log in error?
   std::string error_message; ///< Current error message
+  time_t last_sync_time; ///< Last time the log file was synced (only set in SYNC_METHOD_EVERY_SECOND)
 };
 
 #endif /* PLUGIN_TRANSACTION_LOG_TRANSACTION_LOG_H */

